package com.bwie.gateway.filter;

import com.bwie.common.constants.TokenConstants;
import com.bwie.common.utils.JwtUtils;
import com.bwie.common.utils.StringUtils;
import com.bwie.gateway.config.IgnoreWhiteConfig;
import com.bwie.gateway.utils.GatewayUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cloud.gateway.filter.GatewayFilterChain;
import org.springframework.cloud.gateway.filter.GlobalFilter;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.http.server.reactive.ServerHttpRequest;
import org.springframework.stereotype.Component;
import org.springframework.core.Ordered;
import org.springframework.web.server.ServerWebExchange;
import reactor.core.publisher.Mono;

import java.sql.Time;
import java.util.List;
import java.util.concurrent.TimeUnit;

/**
 * @ClassName:
 * @Description:  鉴权过滤器 验证 token
 * @Author: zhuwenqiang
 * @Date: 2023/7/21
 */
@Component
public class AuthFilter implements GlobalFilter, Ordered {

    @Autowired
    private IgnoreWhiteConfig ignoreWhiteConfig;

    @Autowired
    private StringRedisTemplate stringRedisTemplate;

    /**
     * 过滤方法
     * @param exchange
     * @param chain
     * @return
     */
    @Override
    public Mono<Void> filter(ServerWebExchange exchange, GatewayFilterChain chain) {
        // 验证当前请求是否是白名单请求
        List<String> whites = ignoreWhiteConfig.getWhites();
        // 获取当前请求的URI
        ServerHttpRequest request = exchange.getRequest();
        String path = request.getURI().getPath();
        if (StringUtils.matches(path, whites)) {
            return chain.filter(exchange);
        }
        // token非空验证
        String token = request.getHeaders().getFirst(TokenConstants.TOKEN);
        if (StringUtils.isEmpty(token)) {
            return GatewayUtils.errorResponse(exchange, "token不能为空！");
        }
        // token 合法性验证
        try {
            JwtUtils.parseToken(token);
        } catch (Exception e) {
            return GatewayUtils.errorResponse(exchange, "token不合法！");
        }
        // token 有效性验证
        String userKey = JwtUtils.getUserKey(token);
        Boolean hasKey = stringRedisTemplate.hasKey(TokenConstants.LOGIN_TOKEN_KEY + userKey);
        if (!hasKey) {
            return GatewayUtils.errorResponse(exchange, "token失效！");
        }
        // 每次访问后端请求，自动将token续期15分钟
        stringRedisTemplate.expire(TokenConstants.LOGIN_TOKEN_KEY + userKey, 15, TimeUnit.MINUTES);
        // 放行
        return chain.filter(exchange);
    }

    @Override
    public int getOrder() {
        return 0;
    }

}
